<?php
/* Author: Marton Horvath */
/* Date: 2015 */
/* File: my_phplot.php */

require_once 'phplot.php';

/* Definition of custom error handler */
function e($errno, $errstr, $errfile, $errline) 
{
  switch($errno)
  {
    case E_ERROR: 
      throw new Exception('E_ERROR');
      break;
    case E_WARNING: 
      throw new Exception('E_WARNING - ' . $errstr);
      break;
    case E_PARSE: 
      throw new Exception('E_PARSE');
      break;
    case E_NOTICE: 
      break;
    case E_CORE_ERROR: 
      throw new Exception('E_CORE_ERROR');
      break;
    default: 
      throw new Exception('');
      break;
  }
}
set_error_handler('e');

/* Definition of my_phplot class which extends PHPlot */
class my_phplot extends PHPlot {

  /* Creating array for error messages */
  public $ERROR_MESSAGES = array();

/* Creating array for debug messages */
  public $DEBUG_MESSAGES = array();

  /* Detail of plotting */
  protected $PLOT_DETAIL = 5000;

  public $X_scale_parameter = 2;
  public $Y_scale_parameter = 2;

  public $input1 = '';
  public $input2 = '';
  public $input3 = '';
  public $input4 = '';
  
  /* Constructor for my_phplot class */
  function __construct($width=800, $height=600, $out=NULL, $in=NULL)
  {
    parent::__construct($width, $height, $out, $in);
  }

  /* Set detail of plotting */
  function set_plot_detail($detail) 
  {
    $this->PLOT_DETAIL = $detail;
  }

  /* Print steps to file */
  function print_debug($file) 
  {
    // String to write 
    $current = '';
    // Appending debug messages
    for($i=0; $i<count($this->DEBUG_MESSAGES); $i++)
    {
      $current .= $this->DEBUG_MESSAGES[$i] . "\n";
    }
    // Printig to file
    file_put_contents($file, $current);
  }

  /* Print current plot to file */
  function print_plot_data($file) 
  {
    // String to write 
    $current = '';

    // Appending plot data
      $current .= $this->input1 . "\n";
      $current .= $this->input2 . "\n";
      $current .= $this->input3 . "\n";
      $current .= $this->input4 . "\n";
      $current .= $this->xscale_type . "\n";
      $current .= $this->X_scale_parameter . "\n";
      $current .= $this->plot_min_x . "\n";
      $current .= $this->plot_max_x . "\n";
      $current .= $this->x_tick_anchor . "\n";
      $current .= $this->x_tick_inc_u . "\n";
      $current .= $this->yscale_type . "\n";
      $current .= $this->Y_scale_parameter . "\n";
      $current .= $this->plot_min_y . "\n";
      $current .= $this->plot_max_y . "\n";
      $current .= $this->y_tick_anchor . "\n";
      $current .= $this->y_tick_inc_u . "\n";
      $current .= '!plot_data!' . "\n";

    // Printig to file
    file_put_contents($file, $current);
  }

  /* Specify position of altitude value given by $y */
  function position_in_plot($y)
  {
    if($this->yscale_type == 'reciprocal')
    {
      if(is_nan($y)) return 'error';
      else if( ($y >= 1/$this->plot_max_y || $y <= 1/$this->plot_min_y) && ($y >= $this->plot_min_y && $y <= $this->plot_max_y) ) return 'inside';
      else if( ($y < 1/$this->plot_max_y && $y > 0) || ( $y > 1/$this->plot_min_y && $y < 0) || ($y < $this->plot_min_y || $y > $this->plot_max_y) ) return 'outside';
      else return 'error';
    }
    else
    {
      if(is_nan($y)) return 'error';
      else if( ($y <= $this->plot_max_y) && ($y >= $this->plot_min_y) ) return 'inside';
      else if( ($y > $this->plot_max_y) || ($y < $this->plot_min_y) ) return 'outside';
      else return 'error';
    }    
  }

  /*
   * Returns the data array of a function given by $new_function
   * and calculates boundaries of this function
   */
  function CalculateFunctionData($new_function)
  {

    // Calculation of step of graph draing
    $step = ($this->plot_max_x - $this->plot_min_x) / $this->PLOT_DETAIL;
    // Data array of graph
    $data = array();

    // Calculating graph data and filling data array
    for ($i = $this->plot_min_x ; $i <= $this->plot_max_x; $i = round(($i + $step), 10) )
    {
      
      // Exception in function
      try{$new_function($i);}catch(Exception $e)
      {
        array_push($data, array('', $i,   '' ));
        array_push($this->DEBUG_MESSAGES, "Added to data array: (". $i . ",\t custom_error)");
        continue;
      }
      
      // Error in calculation
      if( $this->position_in_plot($new_function($i)) == 'error')
      {
          array_push($data, array('', $i,   '' ));
          array_push($this->DEBUG_MESSAGES, "Added to data array: (". $i . ",\t error)");
          continue;
      }
      // INF in calculation
      if( $new_function($i) == INF )
      {
          array_push($data, array('', $i,   '' ));
          array_push($this->DEBUG_MESSAGES, "Added to data array: (". $i . ",\t e:INF)");
          continue;
      }
      // -INF in calculation
      if( $new_function($i) == -INF )
      {
          array_push($data, array('', $i,   '' ));
          array_push($this->DEBUG_MESSAGES, "Added to data array: (". $i . ",\t e:-INF)");
          continue;
      }

      // Calcualted coordinate is inside of coordinate system
      if( $this->position_in_plot($new_function($i)) == 'inside')
      {
          array_push($data, array('', $i,   $new_function($i) ));
          array_push($this->DEBUG_MESSAGES, "Added to data array: (". $i . ",\t " . $new_function($i) . ")");
      }
      // Calcualted coordinate is inside of coordinate system
      else if( $this->position_in_plot($new_function($i)) == 'outside' )
      {
          array_push($data, array('', $i,   '' ));
          array_push($this->DEBUG_MESSAGES, "Added to data array: (". $i . ",\t none)");
      }
      else
      {
          // error
      }

    }

    return $data;
  }

  function CalculatePointData($new_function)
  {

    // Calculation of step of graph draing
    $step = ($this->plot_max_x - $this->plot_min_x) / $this->PLOT_DETAIL;
    // Data array of graph
    $data = array(array('', ''));

    // // Calculating graph data and filling data array
    // for ($i = $this->plot_min_x ; $i <= $this->plot_max_x; $i = round(($i + $step), 10) )
    // {
      
    //   // Exception in function
    //   try{$new_function($i);}catch(Exception $e)
    //   {
    //     array_push($data, array('', $i,   '' ));
    //     array_push($this->DEBUG_MESSAGES, "Added to data array: (". $i . ",\t custom_error)");
    //     continue;
    //   }
      
    //   // Error in calculation
    //   if( $this->position_in_plot($new_function($i)) == 'error')
    //   {
    //       array_push($data, array('', $i,   '' ));
    //       array_push($this->DEBUG_MESSAGES, "Added to data array: (". $i . ",\t error)");
    //       continue;
    //   }
    //   // INF in calculation
    //   if( $new_function($i) == INF )
    //   {
    //       array_push($data, array('', $i,   '' ));
    //       array_push($this->DEBUG_MESSAGES, "Added to data array: (". $i . ",\t e:INF)");
    //       continue;
    //   }
    //   // -INF in calculation
    //   if( $new_function($i) == -INF )
    //   {
    //       array_push($data, array('', $i,   '' ));
    //       array_push($this->DEBUG_MESSAGES, "Added to data array: (". $i . ",\t e:-INF)");
    //       continue;
    //   }

    //   // Calcualted coordinate is inside of coordinate system
    //   if( $this->position_in_plot($new_function($i)) == 'inside')
    //   {
    //       array_push($data, array('', $i,   $new_function($i) ));
    //       array_push($this->DEBUG_MESSAGES, "Added to data array: (". $i . ",\t " . $new_function($i) . ")");
    //   }
    //   // Calcualted coordinate is inside of coordinate system
    //   else if( $this->position_in_plot($new_function($i)) == 'outside' )
    //   {
    //       array_push($data, array('', $i,   '' ));
    //       array_push($this->DEBUG_MESSAGES, "Added to data array: (". $i . ",\t none)");
    //   }
    //   else
    //   {
    //       // error
    //   }

    // }

    return $data;
  }

  /*
   * Returns scale type for axis X.
   */
  function GetXScaleType()
  {
      return $this->xscale_type;
  }

  /*
   * Returns scale type for axis Y.
   */
  function GetYScaleType()
  {
      return $this->yscale_type;
  }

  /*
   * This function overrides SetXScaleType function in phplot class
   *
   * Select scale type for the X axis.
   */
  function SetXScaleType($which_xst)
  {
      $this->xscale_type = $this->CheckOption($which_xst, 'linear, logarithm, exponent, power, root, reciprocal, normal, lognormal, tan, atan, tanh, atanh', __FUNCTION__);
      return (boolean)$this->xscale_type;
  }

  /*
   * This function overrides SetYScaleType function in phplot class
   *
   * Select scale type for the Y axis.
   */
  function SetYScaleType($which_yst)
  {
      $this->yscale_type = $this->CheckOption($which_yst, 'linear, logarithm, exponent, power, root, reciprocal, normal, lognormal, tan, atan, tanh, atanh',  __FUNCTION__);
      return (boolean)$this->yscale_type;
  }

  /*
   * This function overrides CalcAxisPositions function in phplot class
   *
   * Calculate X and Y Axis Positions, world coordinates.
   * This needs the min/max x/y range set by CalcPlotAreaWorld.
   * It adjusts or sets x_axis_position and y_axis_position per the data.
   * Empty string means the values need to be calculated; otherwise they
   * are supplied but need to be validated against the World area.
   *
   * Note: This used to be in CalcTranslation, but CalcMargins needs it too.
   * This does not calculate the pixel values of the axes. That happens in
   * CalcTranslation, after scaling is set up (which has to happen after
   * margins are set up).
   *
   * For vertical plots, the X axis defaults to Y=0 if that is inside the plot range, else whichever
   * of the top or bottom that has the smallest absolute value (that is, the value closest to 0).
   * The Y axis defaults to the left edge. For horizontal plots, the axis roles and defaults are switched.
   */
  protected function CalcAxisPositions()
  {
      // Validate user-provided X axis position, or calculate a default if not provided:
      if (isset($this->x_axis_position)) {
          // Force user-provided X axis position to be within the plot range:
          if($this->yscale_type == 'reciprocal')
          {
            $this->x_axis_position = 0;
          }
          else
          {
            $this->x_axis_position = min(max($this->plot_min_y, $this->x_axis_position), $this->plot_max_y);
          }
      } 
      elseif ($this->yscale_type == 'logarithm') {
          // Always use 1 for X axis position on logarithm scale plots.
          $this->x_axis_position = 1;
      } 
      elseif ($this->yscale_type == 'exponent') {
          // Always use 0 for X axis position on exponent scale plots.
          $this->x_axis_position = 1;
      } 
      elseif ($this->yscale_type == 'power') {
          // Always use 0 for X axis position on root scale plots.
          $this->x_axis_position = 0;
      } 
      elseif ($this->yscale_type == 'root') {
          // Always use 0 for X axis position on root scale plots.
          $this->x_axis_position = 1;
      } 
      elseif ($this->yscale_type == 'reciprocal') {
          // Always use 0 for X axis position on reciprocal scale plots.
          $this->x_axis_position = 0;
      }
      elseif ($this->yscale_type == 'normal') {
          // Always use 0 for X axis position on reciprocal scale plots.
          $this->x_axis_position = 0;
      }
      elseif ($this->yscale_type == 'lognormal') {
          // Always use 0 for X axis position on reciprocal scale plots.
          $this->x_axis_position = 0;
      }
      elseif ($this->yscale_type == 'tan') {
          // Always use 0 for X axis position on reciprocal scale plots.
          $this->x_axis_position = 0;
      }
      elseif ($this->yscale_type == 'atan') {
          // Always use 0 for X axis position on reciprocal scale plots.
          $this->x_axis_position = 0;
      }
      elseif ($this->yscale_type == 'tanh') {
          // Always use 0 for X axis position on reciprocal scale plots.
          $this->x_axis_position = 0;
      }
      elseif ($this->yscale_type == 'atanh') {
          // Always use 0 for X axis position on reciprocal scale plots.
          $this->x_axis_position = 0;
      }
      elseif ($this->datatype_swapped_xy || $this->plot_min_y > 0) {
          // Horizontal plot, or Vertical Plot with all Y > 0: Place X axis on the bottom.
          $this->x_axis_position = $this->plot_min_y;
      } 
      elseif ($this->plot_max_y < 0) {
          // Vertical plot with all Y < 0, so place the X axis at the top.
          $this->x_axis_position = $this->plot_max_y;
      } 
      else {
          // Vertical plot range includes Y=0, so place X axis at 0.
          $this->x_axis_position = 0;
      }

      // Validate user-provided Y axis position, or calculate a default if not provided:
      if (isset($this->y_axis_position)) {
          // Force user-provided Y axis position to be within the plot range:
          if($this->xscale_type == 'reciprocal')
          {
            $this->y_axis_position = 0;
          }
          else
          {
            $this->y_axis_position = min(max($this->plot_min_x, $this->y_axis_position), $this->plot_max_x);
          }          
      } 
      elseif ($this->xscale_type == 'logarithm') {
          // Always use 1 for Y axis position on logarithm scale plots.
          $this->y_axis_position = 1;
      } 
      elseif ($this->xscale_type == 'exponent') {
          // Always use 0 for Y axis position on exponent scale plots.
          $this->y_axis_position = 1;
      } 
      elseif ($this->xscale_type == 'power') {
          // Always use 0 for Y axis position on root scale plots.
          $this->y_axis_position = 0;
      } 
      elseif ($this->xscale_type == 'root') {
          // Always use 0 for Y axis position on root scale plots.
          $this->y_axis_position = 1;
      } 
      elseif ($this->xscale_type == 'reciprocal') {
          // Always use 0 for Y axis position on reciprocal scale plots.
          $this->y_axis_position = 0;
      } 
      elseif ($this->xscale_type == 'normal') {
          // Always use 0 for Y axis position on reciprocal scale plots.
          $this->y_axis_position = 0;
      } 
      elseif ($this->xscale_type == 'lognormal') {
          // Always use 0 for Y axis position on reciprocal scale plots.
          $this->y_axis_position = 0;
      } 
      elseif ($this->xscale_type == 'tan') {
          // Always use 0 for Y axis position on reciprocal scale plots.
          $this->y_axis_position = 0;
      } 
      elseif ($this->xscale_type == 'atan') {
          // Always use 0 for Y axis position on reciprocal scale plots.
          $this->y_axis_position = 0;
      }
      elseif ($this->xscale_type == 'tanh') {
          // Always use 0 for Y axis position on reciprocal scale plots.
          $this->y_axis_position = 0;
      }
      elseif ($this->xscale_type == 'atanh') {
          // Always use 0 for Y axis position on reciprocal scale plots.
          $this->y_axis_position = 0;
      }
      elseif (!$this->datatype_swapped_xy || $this->plot_min_x > 0) {
          // Vertical plot, or Horizontal Plot with all X > 0: Place Y axis on left side.
          $this->y_axis_position = $this->plot_min_x;
      } 
      elseif ($this->plot_max_x < 0) {
          // Horizontal plot with all X < 0, so place the Y axis on the right side.
          $this->y_axis_position = $this->plot_max_x;
      } 
      else {
          // Horizontal plot range includes X=0: place Y axis at 0.
          $this->y_axis_position = 0;
      }

      if ($this->GetCallback('debug_scale')) {
          $this->DoCallback('debug_scale', __FUNCTION__, array(
              'x_axis_position' => $this->x_axis_position,
              'y_axis_position' => $this->y_axis_position));
      }

        return TRUE;
  }

  /*
   * This function overrides CalcTranslation function in phplot class
   *
   * Calculates scaling stuff...
   */
  protected function CalcTranslation()
  {
      if ($this->plot_max_x - $this->plot_min_x == 0) { // Check for div by 0
          $this->xscale = 0;
      } else {
          if ($this->xscale_type == 'logarithm') {
              $this->xscale = $this->plot_area_width /
                              (log($this->plot_max_x) - log($this->plot_min_x));
          } 
          else if ($this->xscale_type == 'exponent') {
              $this->xscale = $this->plot_area_width /
                              (exp($this->plot_max_x) - exp($this->plot_min_x) );
          }
          else if ($this->xscale_type == 'power') {
              $this->xscale = $this->plot_area_width /
                              (pow($this->plot_max_x, $this->X_scale_parameter) - pow($this->plot_min_x, $this->X_scale_parameter) );
          }
          else if ($this->xscale_type == 'root') {
              $this->xscale = $this->plot_area_width /
                              (pow($this->plot_max_x, 1/$this->X_scale_parameter) - pow($this->plot_min_x, 1/$this->X_scale_parameter) );
          }
          else if ($this->xscale_type == 'reciprocal') {
              $this->xscale = $this->plot_area_width /
                              (($this->plot_max_x) - ($this->plot_min_x) );
          }
          else if ($this->xscale_type == 'normal') {
              $this->xscale = $this->plot_area_width /
                              (ndst($this->plot_max_x) - ndst($this->plot_min_x) );
          }
          else if ($this->xscale_type == 'lognormal') {
              $this->xscale = $this->plot_area_width /
                              (logndst($this->plot_max_x) - logndst($this->plot_min_x) );
          }
          else if ($this->xscale_type == 'tan') {
              $this->xscale = $this->plot_area_width /
                              (tan($this->plot_max_x) - tan($this->plot_min_x) );
          }
          else if ($this->xscale_type == 'atan') {
              $this->xscale = $this->plot_area_width /
                              (atan($this->plot_max_x) - atan($this->plot_min_x) );
          }
          else if ($this->xscale_type == 'tanh') {
              $this->xscale = $this->plot_area_width /
                              (tanh($this->plot_max_x) - tanh($this->plot_min_x) );
          }
          else if ($this->xscale_type == 'atanh') {
              $this->xscale = $this->plot_area_width /
                              (atanh($this->plot_max_x) - atanh($this->plot_min_x) );
          }
          else {
              $this->xscale = $this->plot_area_width / ($this->plot_max_x - $this->plot_min_x);
          }
      }

      if ($this->plot_max_y - $this->plot_min_y == 0) { // Check for div by 0
          $this->yscale = 0;
      } else {
          if ($this->yscale_type == 'logarithm') {
              $this->yscale = $this->plot_area_height /
                              (log($this->plot_max_y) - log($this->plot_min_y));
          } 
          else if ($this->yscale_type == 'exponent') {
              $this->yscale = $this->plot_area_height /
                              ( exp($this->plot_max_y) - exp($this->plot_min_y) );
          } 
          else if ($this->yscale_type == 'power') {
              $this->yscale = $this->plot_area_height /
                              (pow($this->plot_max_y, $this->Y_scale_parameter) - pow($this->plot_min_y, $this->Y_scale_parameter) );
          }
          else if ($this->yscale_type == 'root') {
              $this->yscale = $this->plot_area_height /
                              (pow($this->plot_max_y, 1/$this->Y_scale_parameter) - pow($this->plot_min_y, 1/$this->Y_scale_parameter) );
          }
          else if ($this->yscale_type == 'reciprocal') {
              $this->yscale = $this->plot_area_height /
                              ( ($this->plot_max_y) - ($this->plot_min_y) );
          } 
          else if ($this->yscale_type == 'normal') {
              $this->yscale = $this->plot_area_height /
                              ( ndst($this->plot_max_y) - ndst($this->plot_min_y) );
          } 
          else if ($this->yscale_type == 'lognormal') {
              $this->yscale = $this->plot_area_height /
                              ( logndst($this->plot_max_y) - logndst($this->plot_min_y) );
          }
          else if ($this->yscale_type == 'tan') {
              $this->yscale = $this->plot_area_height /
                              ( tan($this->plot_max_y) - tan($this->plot_min_y) );
          } 
          else if ($this->yscale_type == 'atan') {
              $this->yscale = $this->plot_area_height /
                              ( atan($this->plot_max_y) - atan($this->plot_min_y) );
          } 
          else if ($this->yscale_type == 'tanh') {
              $this->yscale = $this->plot_area_height /
                              ( tanh($this->plot_max_y) - tanh($this->plot_min_y) );
          } 
          else if ($this->yscale_type == 'atanh') {
              $this->yscale = $this->plot_area_height /
                              ( atanh($this->plot_max_y) - atanh($this->plot_min_y) );
          }
          else {
              $this->yscale = $this->plot_area_height / ($this->plot_max_y - $this->plot_min_y);
          }
      }
      // GD defines x = 0 at left and y = 0 at TOP so -/+ respectively
      if ($this->xscale_type == 'logarithm') {
          $this->plot_origin_x = $this->plot_area[0] - ($this->xscale * log($this->plot_min_x) );
      } 
      else if ($this->xscale_type == 'exponent') {
          $this->plot_origin_x = $this->plot_area[0] - ($this->xscale * exp($this->plot_min_x) );
      } 
      else if ($this->xscale_type == 'power') {
          $this->plot_origin_x = $this->plot_area[0] - ($this->xscale * pow($this->plot_min_x, $this->X_scale_parameter) );
      } 
      else if ($this->xscale_type == 'root') {
          $this->plot_origin_x = $this->plot_area[0] - ($this->xscale * pow($this->plot_min_x, 1/$this->X_scale_parameter) );
      } 
      else if ($this->xscale_type == 'reciprocal') {
          $this->plot_origin_x = $this->plot_area[0] - ($this->xscale * ($this->plot_min_x) );
      } 
      else if ($this->xscale_type == 'normal') {
          $this->plot_origin_x = $this->plot_area[0] - ($this->xscale * ndst($this->plot_min_x) );
      }
      else if ($this->xscale_type == 'lognormal') {
          $this->plot_origin_x = $this->plot_area[0] - ($this->xscale * logndst($this->plot_min_x) );
      }
      else if ($this->xscale_type == 'tan') {
          $this->plot_origin_x = $this->plot_area[0] - ($this->xscale * tan($this->plot_min_x) );
      }
      else if ($this->xscale_type == 'atan') {
          $this->plot_origin_x = $this->plot_area[0] - ($this->xscale * atan($this->plot_min_x) );
      }
      else if ($this->xscale_type == 'tanh') {
          $this->plot_origin_x = $this->plot_area[0] - ($this->xscale * tanh($this->plot_min_x) );
      }
      else if ($this->xscale_type == 'atanh') {
          $this->plot_origin_x = $this->plot_area[0] - ($this->xscale * atanh($this->plot_min_x) );
      }
      else {
          $this->plot_origin_x = $this->plot_area[0] - ($this->xscale * $this->plot_min_x);
      }

      if ($this->yscale_type == 'logarithm') {
          $this->plot_origin_y = $this->plot_area[3] + ($this->yscale * log($this->plot_min_y));
      }  
      else if ($this->yscale_type == 'exponent') {
          $this->plot_origin_y = $this->plot_area[3] + ($this->yscale * exp($this->plot_min_y) );
      } 
      else if ($this->yscale_type == 'power') {
          $this->plot_origin_y = $this->plot_area[3] + ($this->yscale * pow($this->plot_min_y, $this->Y_scale_parameter) );
      } 
      else if ($this->yscale_type == 'root') {
          $this->plot_origin_y = $this->plot_area[3] + ($this->yscale * pow($this->plot_min_y, 1/$this->Y_scale_parameter) );
      } 
      else if ($this->yscale_type == 'reciprocal') {
          $this->plot_origin_y = ($this->plot_area[3] + ($this->yscale * ($this->plot_min_y) ));
      } 
      else if ($this->yscale_type == 'normal') {
          $this->plot_origin_y = $this->plot_area[3] + ($this->yscale * ndst($this->plot_min_y) );
      } 
      else if ($this->yscale_type == 'lognormal') {
          $this->plot_origin_y = $this->plot_area[3] + ($this->yscale * logndst($this->plot_min_y) );
      } 
      else if ($this->yscale_type == 'tan') {
          $this->plot_origin_y = $this->plot_area[3] + ($this->yscale * tan($this->plot_min_y) );
      } 
      else if ($this->yscale_type == 'atan') {
          $this->plot_origin_y = $this->plot_area[3] + ($this->yscale * atan($this->plot_min_y) );
      } 
      else if ($this->yscale_type == 'tanh') {
          $this->plot_origin_y = $this->plot_area[3] + ($this->yscale * tanh($this->plot_min_y) );
      }
      else if ($this->yscale_type == 'atanh') {
          $this->plot_origin_y = $this->plot_area[3] + ($this->yscale * atanh($this->plot_min_y) );
      }  
      else {
          $this->plot_origin_y = $this->plot_area[3] + ($this->yscale * $this->plot_min_y);
      }

      // Convert axis positions to device coordinates:
      $this->y_axis_x_pixels = $this->xtr($this->y_axis_position);
      $this->x_axis_y_pixels = $this->ytr($this->x_axis_position);

      if ($this->GetCallback('debug_scale')) {
          $this->DoCallback('debug_scale', __FUNCTION__, array(
              'xscale' => $this->xscale, 'yscale' => $this->yscale,
              'plot_origin_x' => $this->plot_origin_x, 'plot_origin_y' => $this->plot_origin_y,
              'y_axis_x_pixels' => $this->y_axis_x_pixels,
              'x_axis_y_pixels' => $this->x_axis_y_pixels));
      }

      return TRUE;
  }

  /*
   * This function overrides xtr function in phplot class
   * 
   * Translate X world coordinate into pixel coordinate
   * See CalcTranslation() for calculation of xscale.
   * Note: This function should be 'protected', but is left public for historical reasons.
   * See GetDeviceXY() for a preferred public method.
   */
  function xtr($x_world)
  {
      if ($this->xscale_type == 'logarithm') {
          $x_pixels = $this->plot_origin_x + log($x_world) * $this->xscale ;
      } 
      else if ($this->xscale_type == 'exponent') {
          $x_pixels = $this->plot_origin_x + exp($x_world) * $this->xscale ;
      }
      else if ($this->xscale_type == 'power') {
          $x_pixels = $this->plot_origin_x + pow($x_world, $this->X_scale_parameter) * $this->xscale ;
      }  
      else if ($this->xscale_type == 'root') {
          $x_pixels = $this->plot_origin_x + pow($x_world, 1/$this->X_scale_parameter) * $this->xscale ;
      } 
      else if ($this->xscale_type == 'reciprocal') {
          if(abs($x_world) < 1E-10 )
          {
             $x_pixels = $this->plot_origin_x + 0 * $this->xscale ;
          }
          else
          {
             $x_pixels = $this->plot_origin_x + 1/$x_world * $this->xscale ;
          }
         
      } 
      else if ($this->xscale_type == 'normal') {
          $x_pixels = $this->plot_origin_x + ndst($x_world) * $this->xscale ;
      } 
      else if ($this->xscale_type == 'lognormal') {
          $x_pixels = $this->plot_origin_x + logndst($x_world) * $this->xscale ;
      } 
      else if ($this->xscale_type == 'tan') {
          $x_pixels = $this->plot_origin_x + tan($x_world) * $this->xscale ;
      } 
      else if ($this->xscale_type == 'atan') {
          $x_pixels = $this->plot_origin_x + atan($x_world) * $this->xscale ;
      } 
      else if ($this->xscale_type == 'tanh') {
          $x_pixels = $this->plot_origin_x + tanh($x_world) * $this->xscale ;
      } 
      else if ($this->xscale_type == 'atanh') {
          $x_pixels = $this->plot_origin_x + atanh($x_world) * $this->xscale ;
      } 
      else {
          $x_pixels = $this->plot_origin_x + $x_world * $this->xscale ;
      }
      return round($x_pixels);
  }

  /*
   * This function overrides ytr function in phplot class
   * 
   * Translate Y world coordinate into pixel coordinate.
   * See CalcTranslation() for calculation of yscale.
   * Note: This function should be 'protected', but is left public for historical reasons.
   * See GetDeviceXY() for a preferred public method.
   */
  function ytr($y_world)
  {
      if ($this->yscale_type == 'logarithm') {
          //minus because GD defines y = 0 at top. doh!
          $y_pixels =  $this->plot_origin_y - log($y_world) * $this->yscale ;
      }  
      else if ($this->yscale_type == 'exponent') {
          //minus because GD defines y = 0 at top. doh!
          $y_pixels =  $this->plot_origin_y - exp($y_world) * $this->yscale ;
      } 
      else if ($this->yscale_type == 'power') {
          //minus because GD defines y = 0 at top. doh!
          $y_pixels =  $this->plot_origin_y - pow($y_world, $this->Y_scale_parameter) * $this->yscale ;
      } 
      else if ($this->yscale_type == 'root') {
          //minus because GD defines y = 0 at top. doh!
          $y_pixels =  $this->plot_origin_y - pow($y_world, 1/$this->Y_scale_parameter) * $this->yscale ;
      } 
      else if ($this->yscale_type == 'reciprocal') {
          //minus because GD defines y = 0 at top. doh!
          if(abs($y_world) < 1E-10 )
          {
            $y_pixels =  $this->plot_origin_y - 0 * $this->yscale ;
          }
          else
          {
            $y_pixels =  $this->plot_origin_y - 1/$y_world * $this->yscale ;
          }
      } 
      else if ($this->yscale_type == 'normal') {
          //minus because GD defines y = 0 at top. doh!
          $y_pixels =  $this->plot_origin_y - ndst($y_world) * $this->yscale ;
      } 
      else if ($this->yscale_type == 'lognormal') {
          //minus because GD defines y = 0 at top. doh!
          $y_pixels =  $this->plot_origin_y - logndst($y_world) * $this->yscale ;
      } 
      else if ($this->yscale_type == 'tan') {
          //minus because GD defines y = 0 at top. doh!
          $y_pixels =  $this->plot_origin_y - tan($y_world) * $this->yscale ;
      } 
      else if ($this->yscale_type == 'atan') {
          //minus because GD defines y = 0 at top. doh!
          $y_pixels =  $this->plot_origin_y - atan($y_world) * $this->yscale ;
      } 
      else if ($this->yscale_type == 'tanh') {
          //minus because GD defines y = 0 at top. doh!
          $y_pixels =  $this->plot_origin_y - tanh($y_world) * $this->yscale ;
      } 
      else if ($this->yscale_type == 'atanh') {
          //minus because GD defines y = 0 at top. doh!
          $y_pixels =  $this->plot_origin_y - atanh($y_world) * $this->yscale ;
      } 
      else {
          $y_pixels =  $this->plot_origin_y - $y_world * $this->yscale ;
      }
      return round($y_pixels);
  }

  function set_x_axis_pos($value)
  {
    $this->x_axis_position = $value;
  }
  function set_y_axis_pos($value)
  {
    $this->y_axis_position = $value;
  }

}

?>
